#!/bin/env groovy

//
// Copyright 2018 SAS Institute Inc.
// SAS Campus Drive, Cary, North Carolina 27513, USA.
// All rights reserved.
//

import java.nio.file.*
import java.nio.charset.Charset

@groovy.transform.SourceURI
def SOURCE_URI
def SCRIPT_DIR=new File(SOURCE_URI).parent

// .inc file we get from .inc.orig is not in strict .properties format, needs some adjustments
String propsText = new File("$SCRIPT_DIR/env.inc").getText('UTF-8')
propsText = propsText.replace("\\", "\\\\")

Properties env = new Properties()
env.load(new StringReader(propsText))

int nameArgIdx = java.util.Arrays.asList(args).indexOf('-name')
int destArgIdx = java.util.Arrays.asList(args).indexOf('-destinationpath')
if( nameArgIdx == -1 || args.length < nameArgIdx+2 ) {
    System.err.println "Usage: convert_to_work_group.sh -name <name> -destinationpath <new_path>"
    System.exit 1
}

String BU_NAME=args[nameArgIdx+1]
String BU_NEW_PATH=args[destArgIdx+1]

def FSEP=File.separator


if( ! System.properties['os.name'].toLowerCase().contains('windows') ) {
	def cmd = ["chmod", "2770", "$BU_NEW_PATH"]
	def process = cmd.execute()
	process.waitFor()
	if( process.exitValue() != 0 ) {
		System.err.println process.err.text 
		System.exit 1
	}
}


String OLD_BU_PATH=env.SASCONFIG_MIP_DATA_DIR + FSEP + (BU_NAME=="Public" ? "data" : BU_NAME)

println "Copying old work group data for \"$BU_NAME\" from \"$OLD_BU_PATH\" into \"$BU_NEW_PATH\"..."

if( ! new File(OLD_BU_PATH).isDirectory() ) {
    System.err.println "Old data directory \"$OLD_BU_PATH\" does not exists"
    System.exit 1
}

if( ! new File(BU_NEW_PATH).isDirectory() ) {
    System.err.println "Work group directory \"$BU_NEW_PATH\" does not exists"
    System.exit 1
}

//new AntBuilder().move( todir: BU_NEW_PATH ) {
//  fileset( dir: OLD_BU_PATH )
//}

// we are copying instead of moving so it's easier to retry if something fails
def cmd = System.properties['os.name'].toLowerCase().contains('windows')
        ? "xcopy /e /y".split() + "${OLD_BU_PATH}\\*.*" + BU_NEW_PATH
        : "cp -a".split() + "${OLD_BU_PATH}/." + "$BU_NEW_PATH/"

cmd = cmd as String[]			// seems that "+" converted cmd to Object[]
def process = cmd.execute()

def out = new StringBuffer()
def err = new StringBuffer()
process.consumeProcessOutput( out, err )

process.waitFor()
int ret = process.exitValue()

if( ret != 0 ) {
    System.err.println "Moving work group data failed:"
    System.err.println err
    System.exit 1
}


if( env.OLD_RELEASE.startsWith("2.") && ! env.OLD_RELEASE.startsWith("2.4") ) {

	String econFolder = "$BU_NEW_PATH/input/economics"
	new File(econFolder).eachFile { file ->
		if( ! file.name.endsWith(".sas7bdat") )
			return

		String newEconDirName = file.name.replace(".sas7bdat", "")
		File newEconDir = new File(econFolder, newEconDirName)
		newEconDir.mkdirs()

		println "Moving ${file.name} to ${newEconDirName}..."
		file.renameTo(new File(newEconDir, file.name))
	}
	
	new File("$BU_NEW_PATH/input/functions").mkdirs()
	
	new File("$BU_NEW_PATH/input/risk_data").mkdirs()
	
	String[] RISK_DATA_OBJ_DIRS="pmx cash_flows value_data".split()
	
	RISK_DATA_OBJ_DIRS.each { String dirName ->
		new File("$BU_NEW_PATH/input/risk_data/$dirName").mkdirs()
	}
	
	
	String[] IMPORT_EXPORT_SUBDIRS="logs modeling_systems/logs content_packages/logs".split()
	
	IMPORT_EXPORT_SUBDIRS.each { String dirName ->
		new File("$BU_NEW_PATH/input/import/$dirName").mkdirs()
		new File("$BU_NEW_PATH/output/export/$dirName").mkdirs()
	}
	
	new File("$BU_NEW_PATH/content").mkdirs()

		
	File oldCashFlowsDir = new File("$BU_NEW_PATH/input/cashflows")
	File newCashFlowsDir = new File("$BU_NEW_PATH/input/risk_data/cash_flows")

	if( oldCashFlowsDir.exists() ) {
		oldCashFlowsDir.eachFile() { File file ->  
			file.renameTo(new File(newCashFlowsDir, file.getName()));
		}  
	}
	
	try {
		oldCashFlowsDir.deleteDir()
	}
	catch(Exception e) {
	}
}

new File("$BU_NEW_PATH/output/attributions").mkdirs()

new File("$BU_NEW_PATH/SASCustomCode/sas/risk/modimp").mkdirs()

File prtLibFile = new File(env.SASHOME_SASFOUNDATION_MIP_MACRO_DIR, "rskrpt_print_libraries.sas")
Files.copy(prtLibFile.toPath(), Paths.get("$BU_NEW_PATH/SASCustomCode/${prtLibFile.name}"), StandardCopyOption.REPLACE_EXISTING)


evaluate(new File("$SCRIPT_DIR/migrate_work_group_release.groovy"))

println "Done, please check output above for errors and adjust ownership/permissions on \"$BU_NEW_PATH\" directory if needed"
println "You may also want to remove old data in \"$OLD_BU_PATH\""
