/***********************************************************************
 * Copyright 2017, SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 * NAME:        migrate_path.groovy
 * PURPOSE:     Groovy script to change absolute directory path in SASConfig files during MIP migration
 *
 ***********************************************************************/

@groovy.transform.SourceURI
def SOURCE_URI
def SCRIPT_DIR=new File(SOURCE_URI).parent

// .inc file we get from .inc.orig is not in strict .properties format, needs some adjustments
String propsText = new File("$SCRIPT_DIR/env.inc").getText('UTF-8')
propsText = propsText.replace("\\", "\\\\")

Properties env = new Properties()
env.load(new StringReader(propsText))


def FSEP=File.separator

def MIGRATE_FILE_EXT = ".sas"

int migratedCount = 0

int nameArgIdx = java.util.Arrays.asList(args).indexOf('-name')
if( nameArgIdx == -1 || args.length < nameArgIdx+2 ) {
    System.err.println "Usage: migrate_path.groovy -name <name> -destinationpath <new_path>"
    System.exit 1
}
int destArgIdx = java.util.Arrays.asList(args).indexOf('-destinationpath')
if( destArgIdx == -1 || args.length < destArgIdx+2 ) {
    System.err.println "Usage: migrate_path.groovy -name <name> -destinationpath <new_path>"
    System.exit 1
}

def workGroupName = args[nameArgIdx+1]
def newPath = args[destArgIdx+1]


def OLD_DATA_PATH = env.OLD_RELEASE.startsWith("2.")
    ? env.OLD_APPDATA_PATH + FSEP + (workGroupName=="Public"  ? "data" : workGroupName)
    : env.OLD_RWG_APPDATA_PATH + FSEP + workGroupName + FSEP + "SASModelImplementationPlatform"

def NEW_DATA_PATH = env.NEW_RWG_APPDATA_PATH + FSEP + workGroupName + FSEP + "SASModelImplementationPlatform"
def OLD_CASHFLOW_LIBNAME = "&MI_INPUT_DATA${FSEP}cashflows"
def NEW_CASHFLOW_LIBNAME = "&MI_INPUT_DATA${FSEP}risk_data${FSEP}cash_flows"

def migrateCashflow = env.OLD_RELEASE.startsWith("2.") && ! env.OLD_RELEASE.startsWith("2.4")

try {
    // this requires groovy >= 1.7.1
    new File(args[destArgIdx+1]).traverse(type : groovy.io.FileType.FILES) { file ->
        if( ! file.name.endsWith(MIGRATE_FILE_EXT) )
            return

        def text = file.getText('UTF-8')
		def changed = false

		//Step 1
		//example: /install/cfgsas1/SAS94/SASConfig/Lev1/AppData/SASModelImplementationPlatform/data  ->  /install/RiskTesting/SASConfig/Lev1/AppData/SASRiskWorkGroup/groups/data/SASModelImplementationPlatform
        if( text.contains(OLD_DATA_PATH) ) {
			println "Updating appdata path in ${file.absolutePath}"
			text = text.replace(OLD_DATA_PATH, NEW_DATA_PATH)
			changed = true
        }

		//Step 2 (must be after step 1)
		//example: /install/cfgsas1/SAS94/SASConfig/Lev1/AppData/SASModelImplementationPlatform  ->  /install/RiskTesting/SASConfig/Lev1/AppData/SASModelImplementationPlatform
		if( text.contains(env.OLD_APPDATA_PATH) ) {
			println "Updating mip appdata path in ${file.absolutePath}"
			text = text.replace(env.OLD_APPDATA_PATH, env.SASCONFIG_MIP_DATA_DIR)
			changed = true
		}
		
		//If the migration is 2.x -> 3.x, replace &MI_ROOT/<work group name> with new migrated work group root path such as <SAS config>/<Levx>/AppData/SASRiskWorkGroup/groups/<work group name>/SASModelImplementationPlatform
		//Example: &MI_ROOT/data  ->  /install/RiskTesting/SASConfig/Lev1/AppData/SASRiskWorkGroup/groups/data/SASModelImplementationPlatform
		if(env.OLD_RELEASE.startsWith("2.") && env.NEW_RELEASE.startsWith("3.")){
			def wgName = workGroupName=="Public"  ? "data" : workGroupName
			if( text.contains("&MI_ROOT/"+wgName) ) {
				println "Updating &MI_ROOT/"+wgName+" in ${file.absolutePath}"
				text = text.replace("&MI_ROOT/"+wgName, NEW_DATA_PATH)
				changed = true
			}
		}
		
		if( migrateCashflow && text.contains(OLD_CASHFLOW_LIBNAME) ) {
			println "Updating cashflow path in ${file.absolutePath}"
			text = text.replace(OLD_CASHFLOW_LIBNAME, NEW_CASHFLOW_LIBNAME)
			changed = true
		}

		if( changed ) {
			file.write(text, 'UTF-8')
			migratedCount += 1
		}

    }
}
catch(Exception e) {
    System.err.println "Failed to update appdata path in .sas files:"
    e.printStackTrace()

    // Returning a non-zero return code will force stop the SDW migration (hard failure), so that the customer has a chance to
    // analyze logs and immediately retry the migration.
    System.exit(exitCode)
}
finally {
        println "Migrated $migratedCount .sas files\n"
}
