#!/bin/env groovy

//
// Copyright 2017 SAS Institute Inc.
// SAS Campus Drive, Cary, North Carolina 27513, USA.
// All rights reserved.
//

@groovy.transform.SourceURI
def SOURCE_URI
def SCRIPT_DIR=new File(SOURCE_URI).parent

// .inc file we get from .inc.orig is not in strict .properties format, needs some adjustments
String propsText = new File("$SCRIPT_DIR/env.inc").getText('UTF-8')
propsText = propsText.replace("\\", "\\\\")

Properties env = new Properties()
env.load(new StringReader(propsText))


int nameArgIdx = java.util.Arrays.asList(args).indexOf('-name')
if( nameArgIdx == -1 || args.length < nameArgIdx+2 ) {
    System.err.println "Usage: migrate_work_group_release.groovy -name <name> -destinationpath <new_path>"
    System.exit 1
}


if( !env.OLD_RELEASE.startsWith("2.") && env.OLD_RWG_APPDATA_PATH == env.NEW_RWG_APPDATA_PATH ) {
    println "Old and new paths are the same, no path adjustment needed"
    System.exit(0)
}


println "Running script to migrate path in .sas files..."

evaluate(new File("$SCRIPT_DIR/migrate_path.groovy"))


String sasCmd = new File(env.SASHOME_SASFOUNDATION_DIR, 'sas').path
String sasFile = new File(SCRIPT_DIR, 'migrate_dataset_cube_path.sas').path

// sas script needs config_file_list.cvs in TMP_DIR
new File("${env.TMP_DIR}/config_file_list.csv").text = new File("$SCRIPT_DIR/config_file_list.csv").text

// Create migrate_path_config.csv in TMP_DIR for sas script
String vars = env.OLD_RELEASE.startsWith("2.") 
    ? "${env.NEW_RWG_APPDATA_PATH},${env.OLD_APPDATA_PATH},${env.NEW_RWG_APPDATA_PATH},${env.OLD_APPDATA_PATH},${env.NEW_APPDATA_PATH}"
    : "${env.NEW_RWG_APPDATA_PATH},${env.OLD_RWG_APPDATA_PATH},${env.NEW_RWG_APPDATA_PATH},${env.OLD_APPDATA_PATH},${env.NEW_APPDATA_PATH}" 

new File("${env.TMP_DIR}/migrate_path_config.csv").withWriter('UTF-8') {
    writer -> writer.writeLine "search_dir,old_path,new_path,old_appdata_path,new_appdata_path\n" + vars
} 

println "Running script to migrate path in sas datasets and repair cube descriptors..."

def processBuilder = new ProcessBuilder(sasCmd, "-sysin", sasFile, 
    "-set", "old_release", env.OLD_RELEASE,
    "-set", "work_group", args[nameArgIdx+1])

processBuilder.inheritIO()
def process = processBuilder.start()
process.waitFor()

int ret = process.exitValue()
if( ret ) {
    System.err.println "Failed to migrate path in sas datasets"
    System.exit ret
}
