// (c) 2016, SAS Institute Inc.

jQuery.sap.require("sap.ui.core.mvc.JSView");

jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.TextArea");

jQuery.sap.require("sas.hc.ui.layout.VerticalLayout");

jQuery.sap.require("sas.risk.common.widgets.HelpLabel");

jQuery.sap.require("sas.risk.modimp.common.utils.CommonUtil");
jQuery.sap.require("sas.risk.modimp.implementation.utils.ModelImplementationConstants");

sap.ui.jsview("sas.risk.modimp.implementation.wizard.steps.AdvancedOptionsStep", (function() {
    // pseudo-imports
    var CommonUtil = sas.risk.modimp.common.utils.CommonUtil;
    var ModelImplementationConstants = sas.risk.modimp.implementation.utils.ModelImplementationConstants;

    // vars
    var self;
    var myController;
    var type;
    var rb = CommonUtil.getBundle();

    // controls
    var hpRiskProcOptionsEditor;
    var crossClassVariablesEditor;
    var aggregationLevelsEditor;
    var outputDataSetsEditor;

    /**
     * Set up listeners to update the step status when input changes.
     * @private
     */
    function _configureInputChangeListeners() {

    }

    function _createHPRiskProcOptionsEditor() {
        hpRiskProcOptionsEditor = _createEditor("procHPRiskArguments", "/advancedOptions/procHpriskArguments", 4);

        return _createEditorLayout(
            rb.getText("ModelImplementation.RunInstanceInput.HPRiskProcOptions.txt"),
            hpRiskProcOptionsEditor,
            ModelImplementationConstants.HELP_KEY_ADV_EXEC_PROC_HPRISK
        );
    }

    function _createCrossClassVariablesEditor() {
        crossClassVariablesEditor = _createEditor("crossClassVariables", "/advancedOptions/crossClassVariables", 4, 100000);
        return _createEditorLayout(
            rb.getText("ModelImplementation.RunInstanceInput.CrossClassificationVariables.txt"),
            crossClassVariablesEditor,
            ModelImplementationConstants.HELP_KEY_ADV_EXEC_CROSS_CLASS_VARS,
            "{localModel>/stepRequired}"
        );
    }

    function _createAggregationLevelsEditor() {
        aggregationLevelsEditor = _createEditor("aggregationLevels", "/advancedOptions/aggregationLevels", 4);
        return _createEditorLayout(
            rb.getText("ModelImplementation.RunInstanceInput.AggregationLevels.txt"),
            aggregationLevelsEditor,
            ModelImplementationConstants.HELP_KEY_ADV_EXEC_AGG_LEVELS
        );
    }

    function _createOutputDataSetsEditor() {
        outputDataSetsEditor = _createEditor("outputDataSets", "/advancedOptions/outputDataSets", 6);
        return _createEditorLayout(
            rb.getText("ModelImplementation.RunInstanceInput.OutputDataSets.txt"),
            outputDataSetsEditor,
            ModelImplementationConstants.HELP_KEY_ADV_EXEC_OUTPUT_DATASETS
        );
    }

    function _createEditorLayout(labelText, editor, helpKey, requiredProp) {
        var optionsEditorLabel = new sas.risk.common.widgets.HelpLabel({
            text: labelText,
            library: "sas.risk.modimp",
            key: helpKey,
            required: requiredProp!==undefined ? requiredProp : false
        });
        optionsEditorLabel.addStyleClass("riskInputLabel");

        return new sas.hc.ui.layout.VerticalLayout({
            content: [
                optionsEditorLabel,
                editor
            ]
        }).addStyleClass("riskInputLayout");
    }

    function _createEditor(baseId, modelProperty, rowCount, maxLength) {
        var editor = new sas.hc.m.TextArea({
            width: "50rem",
            value: "{" + modelProperty + "}",
            rows: rowCount,
            maxLength: maxLength ? maxLength : 4000,
            enabled: {
                parts: [
                    {path: "/useDefaultAdvancedOptions"},
                ],
                formatter: function(useDefaultAdvancedOptions) {
                    return !useDefaultAdvancedOptions;
                }
            }
        });
        editor.addStyleClass("mipAdvancedOptionsEditor");

        return editor;
    }

    return {
        // framework functions

        /**
         * Get the associated view controller.
         * @returns {string} The view controller
         */
        getControllerName: function () {
            return "sas.risk.modimp.implementation.wizard.steps.AdvancedOptionsStep";
        },

        /**
         * Implementation of the init function.
         */
        init: function() {
            _configureInputChangeListeners();
        },

        /**
         * Implementation of the create content functionality for this view.
         *
         * @param controller
         * @returns {sas.hc.ui.layout.VerticalLayout}
         */
        createContent: function(controller) {
            self = this;

            myController = controller;

            eventChannelID = this.getViewData().eventChannelID;
            type = this.getViewData().type;
            includeMSA = this.getViewData().includeMSA;

            // configure default option layout
            var defaultOptionLayout = new sas.hc.ui.layout.VerticalLayout({
                width: "100%",
                content: [
                    new sas.hc.m.CheckBox({
                        text: rb.getText("ModelImplementation.RunInstanceInput.UseAdvancedDefaults.txt"),
                        selected: "{/useDefaultAdvancedOptions}",
                        select: function(event) {
                            var selected = event.getParameter("selected");
                            if(selected === true) {
                                self.clearAdvancedOptionsStates();
                                myController.clearAdvancedOptions();
                                myController.loadDefaultValues(true)
                                    .done(function(result){
                                        myController.handleInputChanged();
                                    });
                            }
                        }
                    })
                ]
            });

            // configure step options layout
            var advancedOptionsContent = [];
            advancedOptionsContent.push(_createHPRiskProcOptionsEditor());
            if (type !== ModelImplementationConstants.SCENARIO_TYPE) {
                advancedOptionsContent.push(_createCrossClassVariablesEditor());
            }
            if (type !== ModelImplementationConstants.PORTFOLIO_CUBE_TYPE) {
                advancedOptionsContent.push(_createAggregationLevelsEditor());
                advancedOptionsContent.push(_createOutputDataSetsEditor());
            }
            var optionsLayout = new sas.hc.ui.layout.VerticalLayout({
                width: "100%",
                content: advancedOptionsContent
            }).addStyleClass("mipWizardStepSectionLayout");

            // configure step layout
            var stepLayout = new sas.hc.ui.layout.VerticalLayout({
                width: "100%",
                content: [
                    defaultOptionLayout,
                    optionsLayout
                ]
            }).addStyleClass("mipWizardSectionContainerLayout");

            return stepLayout;
        },

        handleStepSelected: function() {
            if(self.getModel().getProperty("/useDefaultAdvancedOptions") === true) {
                myController.loadDefaultValues(true);
            }
        },

        handleStepRequired: function(){
            if (crossClassVariablesEditor) {
                crossClassVariablesEditor.attachLiveChange(function (e) {
                    var newValue = e.getParameter("newValue");
                    if (!newValue) {
                        crossClassVariablesEditor.setValueState(sap.ui.core.ValueState.Error);
                        crossClassVariablesEditor.setValueStateText(rb.getText("ModelImplementation.AdvancedOptionsStep.crossClassVar.required.txt"));
                    } else {
                        crossClassVariablesEditor.setValueState(sap.ui.core.ValueState.None);
                        crossClassVariablesEditor.setValueStateText("");
                    }
                    myController.handleInputChanged();
                });
            }
        },

        /**
         * Returns true if all validated inputs have no error state.
         * @return {Boolean}
         */
        areInputsValid: function() {
            return !crossClassVariablesEditor || crossClassVariablesEditor.getValueState() === sap.ui.core.ValueState.None;
        },

        // wizard functions

        /**
         * Provides the view a reference to its parent step.
         * @param {sas.risk.modimp.common.widgets.RiskWizardStep} step
         */
        setStep: function(step) {
            myController.setStep(step);
        },

        clearAdvancedOptionsStates: function(){
            if (crossClassVariablesEditor) {
                crossClassVariablesEditor.setValueState(sap.ui.core.ValueState.None);
                crossClassVariablesEditor.setValueStateText("");
            }
        }
    };
}()));