// (c) 2016, SAS Institute Inc.

jQuery.sap.declare("sas.risk.modimp.implementation.wizard.utils.WizardUtil");

jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.Dialog");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.Text");
jQuery.sap.require("sas.hc.ui.layout.HorizontalLayout");
jQuery.sap.require("sas.hc.ui.layout.VerticalLayout");

jQuery.sap.require("sas.risk.common.utils.SASStudioProxy");

jQuery.sap.require("sas.risk.modimp.common.utils.CommonUtil");
jQuery.sap.require("sas.risk.modimp.common.utils.PreferenceManager");
jQuery.sap.require("sas.risk.modimp.common.utils.StaticDataCache");
jQuery.sap.require("sas.risk.modimp.common.widgets.DataSetSelector");
jQuery.sap.require("sas.risk.modimp.common.widgets.EconDataSelector");

jQuery.sap.require("sas.risk.modimp.implementation.utils.ModelImplementationAPI");
jQuery.sap.require("sas.risk.modimp.implementation.utils.ModelImplementationUtil");
jQuery.sap.require("sas.risk.modimp.implementation.wizard.widgets.CreateReadUpdateSelector");
jQuery.sap.require("sas.risk.modimp.implementation.wizard.widgets.ListItemSelector");
jQuery.sap.require("sas.risk.modimp.implementation.wizard.widgets.MethodsSelector");

jQuery.sap.require("sas.hc.ui.core.settings.GlobalSettingsHelper");

/**
 * @name sas.risk.modimp.implementation.wizard.utils.WizardUtil
 */
sas.risk.modimp.implementation.wizard.utils.WizardUtil = (function() {
    // pseudo-imports
    var CommonUtil = sas.risk.modimp.common.utils.CommonUtil;
    var ModelImplementationAPI = sas.risk.modimp.implementation.utils.ModelImplementationAPI;
    var ModelImplementationUtil = sas.risk.modimp.implementation.utils.ModelImplementationUtil;
    var ModelImplementationConstants = sas.risk.modimp.implementation.utils.ModelImplementationConstants;
    var SASStudioProxy = sas.risk.common.utils.SASStudioProxy;
    var StaticDataCache = sas.risk.modimp.common.utils.StaticDataCache;

    // vars
    var rb = CommonUtil.getBundle();

    function _createInvalidItemLayout(nameText, valueText) {
        return new sas.hc.ui.layout.HorizontalLayout({
            content: [
                new sas.hc.m.Label({text: nameText}),//.addStyleClass("mimInputLabel"),
                new sas.hc.m.Label({text: valueText}).addStyleClass("mipValueLabel")
            ]
        }).addStyleClass("mipNameValueLayout");
    }

    function _openEconomicDataSet(data, parentView) {
        if (data.category === "folder") {
            _openEconomicFolder(data.name, parentView);
        } else {
            var id = "sas-risk-modimp-implementation-wizard-utils-WizardUtil-SASStudioLibrary-Dialog";
            var existingDialog = sap.ui.getCore().byId(id);
            if (existingDialog) {
                existingDialog.destroy();
            }

            // instruct user to navigate libraries within SAS Studio and then launch studio. If checkbox to not show is
            // set, then just launch studio
            if (sas.hc.ui.core.settings.GlobalSettingsHelper.getSuppressedMessages().indexOf(id) > -1) {
                ModelImplementationUtil.openDataSet(data);
            } else {
                new sas.hc.m.MessageDialog({
                    id: id,
                    type: sas.hc.m.MessageDialogType.Information,
                    text: rb.getText("ModelImplementation.RunInstanceInput.EconomicLibraryFolderDataSetSelectorDialog.navigate.txt.fmt", data.label),
                    title: rb.getText("ModelImplementation.RunInstanceInput.EconomicLibraryFolderDataSetSelectorDialog.title.fmt", data.label),
                    includeDontShowCheckBox: true,
                    buttonActions: [sas.hc.m.MessageDialogAction.Close]
                }).attachButtonPress(function () {
                    ModelImplementationUtil.openDataSet(data);
                }).open();
            }
        }
    }

    function _openEconomicFolder(economicDataFolderName, parentView) {
        ModelImplementationAPI.loadEconomicFolderDataDetail(economicDataFolderName).done(
            function(details) {
                var id = parentView.getId() + "_selectEconDialog";
                var selectionItems = [];
                _.each(details.files, function(file) {
                    selectionItems.push({fileName: file});
                });

                // SelectDialog does not have close events, so it cannot be destroyed after closing. It is added as a
                // dependent to its parent view to ensure cleanup. Reuse by id to avoid memory leaks.
                var selectionDialog = sap.ui.getCore().byId(id);
                if (!selectionDialog) {
                    selectionDialog = new sas.hc.m.SelectDialog(id, {
                        items: {
                            path: "/selectionItems",
                            template: new sas.hc.m.StandardListItem({
                                title: "{fileName}"
                            })
                        },
                        liveChange: function (evt) {
                            var sValue = evt.getParameter("value");
                            var oFilter = new sap.ui.model.Filter("fileName", sap.ui.model.FilterOperator.Contains, sValue);
                            var oBinding = evt.getSource().getBinding("items");
                            oBinding.filter([oFilter]);
                        },
                        confirm: function (itemSelectionEvent) {
                            var details = selectionDialog._details;		   
                            var selectedItem = itemSelectionEvent.getParameter("selectedItem");
                            var datasetName = selectedItem.getTitle();
                            // handle libref supplied through REST API: MIM-1171; test if it ends with "*"
                            if (details.name.indexOf("*", details.name.length - 1) !== -1) {
                                var libref = details.name.split(".")[0];
                                SASStudioProxy.openTable(libref + "." + datasetName);
                            } else {
                                SASStudioProxy.openFile(details.filePath + datasetName);
                            }
                        }
                    });
                }
				
                parentView.addDependent(selectionDialog);
                selectionDialog._details = details;
                selectionDialog.setTitle(rb.getText("ModelImplementation.RunInstanceInput.EconomicFolderDataSetSelectorDialog.title.fmt", [economicDataFolderName]));
                selectionDialog.setModel(new sas.risk.common.models.RiskJSONModel({selectionItems: selectionItems}));
                selectionDialog._searchField.setFilterMode(true); // hack
                selectionDialog.open();
            }
        ).fail(
            function(restError) {
                CommonUtil.showRESTErrorDialog(restError, rb.getText("errorDialog.title.loadEconomicDataSets.fmt", [economicDataFolderName]));
            }
        );
    }

    return {
        // control creation

        /**
         * Creates a new dataset selector configured for choosing portfolio datasets.
         *
         * @returns {sas.risk.modimp.implementation.wizard.widgets.ListItemSelector}
         */
        createPortfolioDatasetSelector: function(controller, width, itemSelectionCallback) {
            return new sas.risk.modimp.common.widgets.DataSetSelector({
                controller: controller,
                width: width,
                inputLabelText: rb.getText("ModelImplementation.RunInstanceInput.PortfolioDataSelector.txt"),
                // inputValueProperty: "/runInstanceConfig/portfolioDataName",
                datasetBase: "/runInstanceConfig/datasets/portfolio",
                inputValueProperty: "/runInstanceConfig/datasets/portfolio/label",
                inputTooltip: rb.getText("ModelImplementation.RunInstanceInput.PortfolioDataSelector.tooltip"),
                openButtonTooltip: rb.getText("ModelImplementation.RunInstanceInput.PortfolioDataSelector.open.sasStudio.tooltip"),
                nameProperty: "name",
                valueProperty: "filePath",
                selectDialogTitle: rb.getText ('ModelImplementation.RunInstanceInput.PortfolioDataSelectorDialog.title.txt'),
                loadItemsFunction: ModelImplementationAPI.loadPortfolioLibraries, //loadPortfolioData,
                loadItemDetailFunction: ModelImplementationAPI.loadDataSetsForLibrary,
                itemSelectionProperty: "/selectedPortfolioFilePath",
                itemSelectionCallback: itemSelectionCallback,
                errorDialogTitle: rb.getText("errorDialog.title.loadPortfolioData.txt"),
                required: true
            });
        },

        /**
         * Creates a new dataset selector configured for choosing economic datasets.
         *
         * @returns {sas.risk.modimp.implementation.wizard.widgets.ListItemSelector}
         */
        createEconomicDatasetSelector: function(controller, width, itemSelectionCallback) {
            return new sas.risk.modimp.common.widgets.EconDataSelector({
                controller: controller,
                width: width,
                inputLabelText: rb.getText("ModelImplementation.RunInstanceInput.EconomicDataSelector.txt"),
                datasetBase: "/runInstanceConfig/datasets/economic",
                inputValueProperty: "/runInstanceConfig/datasets/economic/label",
                inputTooltip: rb.getText("ModelImplementation.RunInstanceInput.EconomicDataSelector.tooltip"),
                viewDataFunction: _openEconomicDataSet,
                openButtonTooltip: rb.getText("ModelImplementation.RunInstanceInput.EconomicDataSelector.open.sasStudio.tooltip"),
                nameProperty: "name",
                valueProperty: "filePath",
                selectDialogTitle: rb.getText ('ModelImplementation.RunInstanceInput.EconomicDataSelectorDialog.title.txt'),
                loadItemsFunction: ModelImplementationAPI.loadEconomicDataLibraries, //loadEconomicData,
                loadItemDetailFunction: ModelImplementationAPI.loadDataSetsForLibrary,
                itemSelectionProperty: "/selectedEconomicDataFilePath",
                itemSelectionCallback: itemSelectionCallback,
                errorDialogTitle: rb.getText("errorDialog.title.loadEconomicData.txt"),
                required: true
            });
        },

        /**
         * Creates a numeraire selector configured for choosing target currencies
         *
         * @returns {sas.risk.modimp.implementation.wizard.widgets.ListItemSelector}
         */
        createNumeraireSelector: function(controller, width) {
            var selectionCallback = function() {
                var choice = listItemSelector.getSelectedItem();
                if (choice) {
                    sas.risk.modimp.common.utils.PreferenceManager.saveNumeraireChoice(choice);
                }
                controller.handleInputChanged();
            };
            var loadItemsFunction = function() {
                var deferred = $.Deferred();
                StaticDataCache.getCurrencies(function(currencies) {
                    currencies ? deferred.resolve(currencies) : deferred.reject(null);
                });
                return deferred;
            };
            var listItemSelector = new sas.risk.modimp.implementation.wizard.widgets.ListItemSelector({
                controller: controller,
                preventOpen: true,
                width: width,
                inputLabelText: rb.getText("ModelImplementation.RunInstanceInput.NumeraireSelector.txt"),
                inputValueProperty: "/runInstanceConfig/outputCurrency/label",
                inputTooltip: rb.getText("ModelImplementation.RunInstanceInput.NumeraireSelector.tooltip"),
                selectDialogTitle: rb.getText ('ModelImplementation.RunInstanceInput.NumeraireSelectorDialog.title.txt'),
                loadItemsFunction: loadItemsFunction,
                nameProperty: "label",
                valueProperty: "id",
                itemSelectionProperty: "/runInstanceConfig/outputCurrency/id",
                itemSelectionCallback: selectionCallback,
                errorDialogTitle: rb.getText("errorDialog.title.loadNumeraire.txt"),
                required: true
            });
            return listItemSelector;
        },

        /**
         * Creates a program selector for the given programType
         * @param programType - type of program
         * @param controller
         * @param inputWidth
         * @param cellWidth
         * @param navChecker (optional) to detect dirty state changes
         */
        createProgramSelector: function(programType, controller, inputWidth, cellWidth, navChecker) {

            // (optional) called when a selection is made from the selection view. used here to retain last choice as a preference
            // MIM-2112
            // var selectionCallback = function() {
            //     // Note that choice can be null. This clears the preference.
            //     var choice = programSelector.getSelectedItem();
            //     sas.risk.modimp.common.utils.PreferenceManager.saveProgramChoice(programType, choice);
            // };

            // supplies content to selection dialog
            var getSelectionView = function(getSelectedItem) {
                var view = sas.risk.modimp.implementation.utils.ViewManager.loadSearchPrograms({
                    programType: programType,
                    configureForSelectionOnly: true,
                    multiSelection: false,
                    showTableOptions: false,
                    selectedPrograms: [getSelectedItem()],
                    editWithDialog: true
                });

                // map view-specific selection function to a generic name callable inside the selector (i.e. interface)
                view.selectionFunction = view.getSelectedPrograms;
                return view;
            };

            // creates the appropriate view/new dialog
            var getCreateUpdateDialog = function(id, afterCloseCallback) {
                programSelector.getProgramEditorDialogProps = function() {
                    return {
                        programId: id,
                        programType: programType,
                        afterCloseCallback: afterCloseCallback,
                        navChecker: navChecker
                    };
                };
                var dialog = sap.ui.jsfragment("sas.risk.modimp.implementation.views.ProgramEditorDialog", programSelector);
                return dialog;
            };

            var programSelector = new sas.risk.modimp.implementation.wizard.widgets.CreateReadUpdateSelector({
                controller: controller,
                selectedItemPath: "/" + programType, // e.g. /postExecutionProgram
                checkBoxSelectionProperty: "/" + programType + "/selected", // e.g. "/postExecutionProgram/selected"
                width: inputWidth,
                cellWidth: cellWidth,
                inputLabelText: rb.getText(programType + ".ModelImplementation.RunInstanceInput.ProgramSelector.txt"),
                inputTooltip: rb.getText(programType + ".select.tooltip.txt"),
                openButtonTooltip: rb.getText(programType + ".ModelImplementation.RunInstanceInput.SelectorDialog.open.tooltip"),
                createButtonTooltip: rb.getText(programType + ".ModelImplementation.RunInstanceInput.SelectorDialog.create.tooltip"),
                itemSelectionCallback: null, // MIM-2112 selectionCallback,
                selectionDialogTitle: rb.getText(programType + ".select.dialog.title.txt"),
                getCreateUpdateDialog: getCreateUpdateDialog,
                getSelectionView: getSelectionView,
                required: false
            });
            return programSelector;
        },

        /**
         * Creates a time bucket selector
         *
         * @returns {sas.risk.modimp.implementation.wizard.widgets.CreateReadUpdateSelector}
         */
        createTimeBucketSelector: function(controller, inputWidth, cellWidth) {

            // supplies content to selection dialog
            var getSelectionView = function(getSelectedItem) {
                var view = sas.risk.modimp.implementation.utils.ViewManager.loadSearchTimeBucketSchemes({
                    configureForSelectionOnly: true,
                    multiSelection: false,
                    showTableOptions: false,
                    selectedPrograms: [getSelectedItem()]
                });

                // map view-specific selection function to a generic name callable inside the selector (i.e. interface)
                view.selectionFunction = view.getSelectedTimeBucketSchemes;
                return view;
            };

            // creates the appropriate view/new dialog
            var getCreateUpdateDialog = function(id, afterCloseCallback) {
                timeBucketSelector.getTimeBucketEditorDialogProps = function() {
                    return {
                        timeBucketId: id,
                        // programType: self.selectorData.programType,
                        afterCloseCallback: afterCloseCallback
                    };
                };
                var dialog = sap.ui.jsfragment("sas.risk.modimp.implementation.views.TimeBucketEditorDialog", timeBucketSelector);
                return dialog;
            };

            var timeBucketSelector = new sas.risk.modimp.implementation.wizard.widgets.CreateReadUpdateSelector({
                controller: controller,
                selectedItemPath: "/runInstanceConfig/cashFlowAnalysis/timeBucketScheme",
                width: inputWidth,
                cellWidth: cellWidth,
                inputLabelText: rb.getText("timeBucketScheme.ModelImplementation.RunInstanceInput.TimeBucketSelector.txt"),
                inputTooltip: rb.getText("timeBucketScheme.select.tooltip.txt"),
                openButtonTooltip: rb.getText("timeBucketScheme.ModelImplementation.RunInstanceInput.SelectorDialog.open.tooltip"),
                createButtonTooltip: rb.getText("timeBucketScheme.ModelImplementation.RunInstanceInput.SelectorDialog.create.tooltip"),
                itemSelectionCallback: null,
                selectionDialogTitle: rb.getText("timeBucketScheme.ModelImplementation.RunInstanceInput.TimeBucketSelector.SelectDialogTitle.txt"),
                getCreateUpdateDialog: getCreateUpdateDialog,
                getSelectionView: getSelectionView,
                required: true,
                labelRequired: false
            });
            return timeBucketSelector;
        },

        /**
         * Creates a methods selector for the given programType
         *
         * @returns {sas.risk.modimp.implementation.wizard.widgets.MethodsSelector}
         */
        createMethodsSelector: function(selectorData) {
            var methodsSelector = new sas.risk.modimp.implementation.wizard.widgets.MethodsSelector(selectorData);
            return methodsSelector;
        },

        /**
         * Creates a new dataset selector configured for choosing economic simulation datasets.
         *
         * @returns {sas.risk.modimp.implementation.wizard.widgets.ListItemSelector}
         */
        createEconomicSimulationDatasetSelector: function(controller, width, itemSelectionCallback) {
            return new sas.risk.modimp.common.widgets.DataSetSelector({
                controller: controller,
                width: width,
                itemSelectionCallback: itemSelectionCallback,
                inputLabelText: rb.getText("ModelImplementation.RunInstanceInput.EconomicSimulationDataSelector.txt"),
                inputValueProperty: "/runInstanceConfig/datasets/economicSimulation/label",
                datasetBase: "/runInstanceConfig/datasets/economicSimulation",
                inputTooltip: rb.getText("ModelImplementation.RunInstanceInput.EconomicSimulationDataSelector.tooltip"),
                openButtonTooltip: rb.getText("ModelImplementation.RunInstanceInput.EconomicSimulationDataSelector.open.sasStudio.tooltip"),
                nameProperty: "name",
                valueProperty: "filePath",
                selectDialogTitle: rb.getText ('ModelImplementation.RunInstanceInput.EconomicSimulationDataSelectorDialog.title.txt'),
                loadItemsFunction: ModelImplementationAPI.loadEconSimulationDataLibraries, //loadEconomicSimulationData,
                loadItemDetailFunction: ModelImplementationAPI.loadDataSetsForLibrary,
                itemSelectionProperty: "/selectedEconomicSimulationDataFilePath",
                errorDialogTitle: rb.getText("errorDialog.title.loadEconomicSimulationData.txt"),
                required: true
            });
        },

        /**
         * Creates a new dataset selector configured for choosing portfolio datasets.
         *
         * @returns {sas.risk.modimp.implementation.wizard.widgets.ListItemSelector}
         */
        createCounterpartyDatasetSelector: function(controller, width, itemSelectionCallback, isRequired, cellWidth) {
            return new sas.risk.modimp.common.widgets.DataSetSelector({
                controller: controller,
                width: width,
                cellWidth: cellWidth,
                inputLabelText: rb.getText("ModelImplementation.RunInstanceInput.CounterpartyDataSelector.txt"),
                inputValueProperty: "/counterpartyData/counterparty/label",
                datasetBase: "/counterpartyData/counterparty",
                inputTooltip: rb.getText("ModelImplementation.RunInstanceInput.CounterpartyDataSelector.tooltip"),
                openButtonTooltip: rb.getText("ModelImplementation.RunInstanceInput.CounterpartyDataSelector.open.sasStudio.tooltip"),
                nameProperty: "name",
                valueProperty: "filePath",
                selectDialogTitle: rb.getText ('ModelImplementation.RunInstanceInput.CounterpartyDataSelectorDialog.title.txt'),
                loadItemsFunction: ModelImplementationAPI.loadCounterpartyDataLibraries, //loadCounterpartyData,
                loadItemDetailFunction: ModelImplementationAPI.loadDataSetsForLibrary,
                itemSelectionProperty: "/counterpartyData/filePath",
                itemSelectionCallback: itemSelectionCallback,
                errorDialogTitle: rb.getText("errorDialog.title.loadCounterpartyData.txt"),
                required: isRequired,
                checkBoxSelectionProperty: "/counterpartyData/selected"
            });
        },

        /**
         * Creates a new dataset selector configured for choosing portfolio datasets.
         *
         * @returns {sas.risk.modimp.implementation.wizard.widgets.ListItemSelector}
         */
        createMitigationDatasetSelector: function(controller, width, itemSelectionCallback, isRequired, cellWidth) {
            return new sas.risk.modimp.common.widgets.DataSetSelector({
                controller: controller,
                width: width,
                cellWidth: cellWidth,
                inputLabelText: rb.getText("ModelImplementation.RunInstanceInput.MitigationDataSelector.txt"),
                inputValueProperty: "/mitigationData/mitigation/label",
                datasetBase: "/mitigationData/mitigation",
                inputTooltip: rb.getText("ModelImplementation.RunInstanceInput.MitigationDataSelector.tooltip"),
                openButtonTooltip: rb.getText("ModelImplementation.RunInstanceInput.MitigationDataSelector.open.sasStudio.tooltip"),
                nameProperty: "name",
                valueProperty: "filePath",
                selectDialogTitle: rb.getText ('ModelImplementation.RunInstanceInput.MitigationDataSelectorDialog.title.txt'),
                loadItemsFunction: ModelImplementationAPI.loadMitigationDataLibraries,
                loadItemDetailFunction: ModelImplementationAPI.loadDataSetsForLibrary,
                itemSelectionProperty: "/mitigationData/filePath",
                itemSelectionCallback: itemSelectionCallback,
                errorDialogTitle: rb.getText("errorDialog.title.loadMitigationData.txt"),
                required: isRequired,
                checkBoxSelectionProperty: "/mitigationData/selected"
            });
        },

        /**
         * Creates a new dataset selector configured for choosing actuals dataset in backtesting.
         *
         * @returns {sas.risk.modimp.implementation.wizard.widgets.ListItemSelector}
         */
        createBacktestingDatasetSelector: function(controller, width, itemSelectionCallback, cellWidth) {
            return new sas.risk.modimp.common.widgets.DataSetSelector({
                controller: controller,
                width: width,
                cellWidth: cellWidth,
                inputLabelText: rb.getText("ModelImplementation.RunInstanceInput.ActualsDataSet.txt"),
                inputValueProperty: "/runInstanceConfig/datasets/actuals/label",
                datasetBase: "/runInstanceConfig/datasets/actuals",
                inputTooltip: rb.getText("ModelImplementation.RunInstanceInput.ActualsDataSet.tooltip"),
                openButtonTooltip: rb.getText("ModelImplementation.RunInstanceInput.ActualsDataSet.open.sasStudio.tooltip"),
                nameProperty: "name",
                valueProperty: "filePath",
                selectDialogTitle: rb.getText ('ModelImplementation.RunInstanceInput.ActualsDataSet.title.txt'),
                loadItemsFunction: ModelImplementationAPI.loadBacktestingLibraries, //loadMitigationData,
                itemSelectionCallback: itemSelectionCallback,
                errorDialogTitle: rb.getText("errorDialog.title.loadActualsData.txt"),
                required: true
            });
        },

        // validation

        isDebugOptionsComplete: function(debugOptions) {
            var maxPrintDefined = !!(debugOptions.maxPrint && ModelImplementationUtil.isValidMaximumTraceCountValue(debugOptions.maxPrint));
            var levelDefined = !!debugOptions.level;
            var allModelsAndGroupsSelected = !!debugOptions.allModelsAndGroups;
            var instrumentIDsValid = !!(!debugOptions.instIds || ModelImplementationUtil.isValidInstrumentIDString(debugOptions.instIds));
            var modelGroupsSelected = !!debugOptions.modelGroupIds && debugOptions.modelGroupIds.length > 0;
            var modelsSelected = !!debugOptions.modelIds && debugOptions.modelIds.length > 0;
            var postProcessMethodsSelected = !!debugOptions.postProcessMethodIds && debugOptions.postProcessMethodIds.length > 0;

            return maxPrintDefined && levelDefined && instrumentIDsValid && (allModelsAndGroupsSelected || modelGroupsSelected || modelsSelected || postProcessMethodsSelected);
        },

        isDebugOptionsPartiallyComplete: function(debugOptions) {
            var maxPrintDefined = !!debugOptions.maxPrint;
            var maxPrintValid = !!(!debugOptions.maxPrint || ModelImplementationUtil.isValidMaximumTraceCountValue(debugOptions.maxPrint));
            var levelDefined = !!debugOptions.level;
            var allModelsAndGroupsSelected = !!debugOptions.allModelsAndGroups;
            var instrumentIDsDefined = !!debugOptions.instIds;
            var instrumentIDsValid = !!(!debugOptions.instIds || ModelImplementationUtil.isValidInstrumentIDString(debugOptions.instIds));
            var modelGroupsSelected = !!debugOptions.modelGroupIds && debugOptions.modelGroupIds.length > 0;
            var modelsSelected = !!debugOptions.modelIds && debugOptions.modelIds.length > 0;
            var postProcessMethodsSelected = !!debugOptions.postProcessMethodIds && debugOptions.postProcessMethodIds.length > 0;

            var isComplete = !!(maxPrintDefined && maxPrintValid && levelDefined && (!instrumentIDsDefined || instrumentIDsValid) && (allModelsAndGroupsSelected || modelGroupsSelected || modelsSelected || postProcessMethodsSelected));

            return isComplete !== true && (maxPrintDefined || levelDefined || instrumentIDsDefined || allModelsAndGroupsSelected || modelGroupsSelected || modelsSelected || postProcessMethodsSelected);
        },


        // utilities

        processWizardFinish: function(parentView, stepPrompts, confirmFunction) {
            var promptString = "";
            _.each(stepPrompts, function(stepPrompt) {
                if(stepPrompt && stepPrompt.promptText) {
                    promptString += (stepPrompt.promptText + "\n\n");
                }
            });

            if(promptString.length > 0) {
                promptString += rb.getText("proceed.confirmation.txt");

                CommonUtil.showConfirmationDialog(
                    parentView,
                    rb.getText("warning.txt"),
                    promptString,
                    null,
                    function() {
                        confirmFunction();
                    }
                );
            }
            else {
                confirmFunction();
            }
        },

        /**
         * Produces a full run configuration by cloning the baseRunConfiguration and
         * adding whatever optional attributes have been specified to it.
         * @param {Object} baseRunConfiguration
         * @param {sas.risk.common.models.RiskJSONModel} model
         * @param {Boolean} isEditMode
         * @returns {Object} the full run configuration
         */
        createFullRunConfiguration: function(baseRunConfiguration, model, isEditMode) {
            var fullRunConfiguration = $.extend(true, {}, baseRunConfiguration);

            // set the resubmit flag
            fullRunConfiguration.reSubmit = isEditMode;

            if (fullRunConfiguration.datasets) {
                var fixDataSetFn = function(dsType) {
                    if (fullRunConfiguration.datasets[dsType] && fullRunConfiguration.datasets[dsType].name) {
                        var name = fullRunConfiguration.datasets[dsType].name;
                        var libref = fullRunConfiguration.datasets[dsType].libref;
                        var category = fullRunConfiguration.datasets[dsType].category;
                        delete fullRunConfiguration.datasets[dsType];
                        var results = { name: name };
                        if (libref) {
                            results.libref = libref;
                        }
                        if (category) {
                            results.category = category;
                        }
                        fullRunConfiguration.datasets[dsType] = results;
                    }
                };
                fixDataSetFn("economic");
                fixDataSetFn("portfolio");
                fixDataSetFn("actuals");
                fixDataSetFn("economicSimulation");

                // If counterparty data is not set, don't add it to the config object.
                var counterpartyData = model.getProperty("/counterpartyData");
                if(counterpartyData && counterpartyData.selected && counterpartyData.counterparty && counterpartyData.counterparty.name) {
                    fullRunConfiguration.datasets.counterparty = {name: counterpartyData.counterparty.name, libref: counterpartyData.counterparty.libref};
                } else {
                    delete fullRunConfiguration.datasets.counterparty;
                    delete fullRunConfiguration.scoringModelGroupMap;
                }

                // If mitigation data is not set, don't add it to the config object.
                var mitigationData = model.getProperty("/mitigationData");
                if(mitigationData && mitigationData.selected && mitigationData.mitigation && mitigationData.mitigation.name) {
                    fullRunConfiguration.datasets.mitigation = {name: mitigationData.mitigation.name, libref: mitigationData.mitigation.libref};
                } else {
                    delete fullRunConfiguration.datasets.mitigation;
                    delete fullRunConfiguration.mitigationMap;
                }

                if (fullRunConfiguration.datasets.riskDataMaps) {
                    if (0 === fullRunConfiguration.datasets.riskDataMaps.length) {
                        delete fullRunConfiguration.datasets.riskDataMaps;
                    } else {
                        _.each(fullRunConfiguration.datasets.riskDataMaps, function(rdMap) {
                            if (rdMap.dataset) {
                                var name = rdMap.dataset.name;
                                var libref = rdMap.dataset.libref;
                                delete rdMap;
                                rdMap.dataset = {name: name, libref: libref};
                            }
                            if (rdMap.riskDataObject) {
                                var rdoName = rdMap.riskDataObject.name;
                                var rdoType = rdMap.riskDataObject.type;
                                delete rdMap.riskDataObject;
                                rdMap.riskDataObject = {name: rdoName, type: rdoType};
                            }
                        });
                    }
                }
            }

            // prune extraneous data from maps
            var fixMapFn = function(mapType) {
                if (_.isEmpty(fullRunConfiguration[mapType])) {
                    delete fullRunConfiguration[mapType];
                } else if (fullRunConfiguration[mapType] && fullRunConfiguration[mapType].id) {
                    var id = fullRunConfiguration[mapType].id;
                    var kind = fullRunConfiguration[mapType].kind;
                    delete fullRunConfiguration.datasets[mapType];
                    var results = {id: id};
                    if (kind) {
                        results.kind = kind;
                    }
                    fullRunConfiguration[mapType] = results;
                }
            };
            fixMapFn("modelGroupMap");
            fixMapFn("scoringModelGroupMap");
            fixMapFn("mitigationMap");

            // If number of simulations is not set, don't add it to the config object. Midtier
            // translates this property when unset to 0, which is an invalid value and will
            // result in an error.
            var numberSimulations = model.getProperty("/numberSimulations");
            if(numberSimulations) {
                fullRunConfiguration.numberSimulations = numberSimulations;
            }

            // remove extraneous object attributes from PPMs and CMs
            if(fullRunConfiguration.postProcessMethods) {
                var optimizedList = [];
                _.each(fullRunConfiguration.postProcessMethods, function (postProcess) {
                    optimizedList.push({id: postProcess.id, sequence: postProcess.sequence});
                });
                fullRunConfiguration.postProcessMethods = optimizedList;
            }

            if(fullRunConfiguration.computedMethods) {
                var optimizedList = [];
                _.each(fullRunConfiguration.computedMethods, function (computedMethod) {
                    optimizedList.push({id: computedMethod.id, sequence: computedMethod.sequence});
                });
                fullRunConfiguration.computedMethods = optimizedList;
            }

            //if includeAllOutputVariables flag is set them clear the outputVariables from runInstanceConfig (MIM-1629)
            if(model.getProperty("/runInstanceConfig/includeAllOutputVariables")){
                delete fullRunConfiguration.outputVariables;
            }

            //Since in midtier side we don't have granular way to differentiate different scenario runs and the differentiation is done based on the REST payload, therefore cleanup irrelevant data before sending to server.
            //If not a backtesting run, then don't include the actuals dataset.
            if(!model.getProperty("/includeBackTesting")){
                delete fullRunConfiguration.datasets.actuals;
            }
            //if not a backtesting run or not set to compute backtesting metrics then remove the entry (/backtestingMetrics) from payload since if present means it is assumed to be the compute backtesting metrics scenario
            if(model.getProperty("/includeBackTesting") && model.getProperty("/calculateBackTestingMetrics")) {
                var optimizedList = [];
                _.each(fullRunConfiguration.backtestingMetrics.metrics, function (metrics) {
                    optimizedList.push({id: metrics.id});
                });
                fullRunConfiguration.backtestingMetrics.metrics = optimizedList;
            } else {
                delete fullRunConfiguration.backtestingMetrics;
            }

            // If pre execution program is not set, don't add it to the config object.
            var preExecutionProgram = model.getProperty("/preExecutionProgram");
            if (preExecutionProgram && preExecutionProgram.selected && preExecutionProgram.name) {
                // assign individually because preExecutionProgram also has a selected attribute which we do not want to send
                fullRunConfiguration.preExecutionProgram = {};
                fullRunConfiguration.preExecutionProgram.name = preExecutionProgram.name;
                fullRunConfiguration.preExecutionProgram.id = preExecutionProgram.id;
            }

            // If post execution program is not set, don't add it to the config object.
            var postExecutionProgram = model.getProperty("/postExecutionProgram");
            if (postExecutionProgram && postExecutionProgram.selected && postExecutionProgram.name) {
                // assign individually because postExecutionProgram also has a selected attribute which we do not want to send
                fullRunConfiguration.postExecutionProgram = {};
                fullRunConfiguration.postExecutionProgram.name = postExecutionProgram.name;
                fullRunConfiguration.postExecutionProgram.id = postExecutionProgram.id;
            }

            // If debug options are not properly set, don't add to the config object.
            var debugOptions = model.getProperty("/debugOptions");
            if(model.getProperty("/runInstanceConfig/debugOn") === true) {
                if(this.isDebugOptionsComplete(debugOptions)) {
                    fullRunConfiguration.debugOptions = debugOptions;
                }
            }

            // apply advanced execution options if set
            var advancedOptions = model.getProperty("/advancedOptions");
            if(model.getProperty("/useDefaultAdvancedOptions") === false) {
                fullRunConfiguration.advancedOptions = {};
                fullRunConfiguration.advancedOptions.procHpriskArguments = advancedOptions.procHpriskArguments ? advancedOptions.procHpriskArguments : "";
                if(baseRunConfiguration.type !== ModelImplementationConstants.SCENARIO_TYPE) {
                    fullRunConfiguration.advancedOptions.crossClassVariables = advancedOptions.crossClassVariables ? advancedOptions.crossClassVariables : "";
                }
                if (baseRunConfiguration.type !== ModelImplementationConstants.PORTFOLIO_CUBE_TYPE) {
                    fullRunConfiguration.advancedOptions.aggregationLevels = advancedOptions.aggregationLevels ? advancedOptions.aggregationLevels : "";
                    fullRunConfiguration.advancedOptions.outputDataSets = advancedOptions.outputDataSets ? advancedOptions.outputDataSets : "";
                }
            }

            // note: must explicitly check for === true with cashFlowAnalysisActive because it can also evaluate to true when it has object properties
            if (model.getProperty("/cashFlowAnalysisVisible") && model.getProperty("/cashFlowAnalysisActive") === true
                && !_.isEmpty(fullRunConfiguration.cashFlowAnalysis.timeBucketScheme)) {
                // exclude unnecessary data
                delete fullRunConfiguration.cashFlowAnalysis.durationName;
                if (fullRunConfiguration.cashFlowAnalysis.timeBucketScheme) {
                    var timeBucketId = fullRunConfiguration.cashFlowAnalysis.timeBucketScheme.id;
                    delete fullRunConfiguration.cashFlowAnalysis.timeBucketScheme;
                    fullRunConfiguration.cashFlowAnalysis.timeBucketScheme = {id: timeBucketId};
                }
                //if includeAllCashFlowLegs flag is set them clear the cashFlowAnalysis from runInstanceConfig
                if(model.getProperty("/runInstanceConfig/cashFlowAnalysis/includeAllCashFlowLegs")){
                    delete fullRunConfiguration.cashFlowAnalysis.cashFlowLegs;
                }
            } else {
                // analyze cash flow is not selected, so don't send any data
                delete fullRunConfiguration.cashFlowAnalysis;
            }

            return fullRunConfiguration;
        },

        /**
         *
         * @param runInstance
         * @param isSourceCube
         * @param parentView
         */
        handleInvalidRunFields: function(runInstance, isSourceCube, parentView) {
            var invalidDatasets = {};

            if(runInstance.validationState && runInstance.validationState.invalidDatasets) {
                for(var attrname in runInstance.validationState.invalidDatasets) {
                    invalidDatasets[attrname] = runInstance.validationState.invalidDatasets[attrname];
                }
            }
            if(runInstance.sourceCube && runInstance.sourceCube.validationState && runInstance.sourceCube.validationState.invalidDatasets) {
                for(var attrname in runInstance.sourceCube.validationState.invalidDatasets) {
                    invalidDatasets[attrname] = runInstance.sourceCube.validationState.invalidDatasets[attrname];
                }
            }

            if(_.isEmpty(invalidDatasets)) {
                return;
            }

            var dialogMessageLayoutContent = [];
            if(invalidDatasets.actuals) {
                dialogMessageLayoutContent.push(_createInvalidItemLayout(rb.getText("ModelImplementation.RunInstanceInput.ActualsDataSet.txt"), invalidDatasets.actuals.label));
            }
            if(invalidDatasets.riskDataMaps) {
                _.each(invalidDatasets.riskDataMaps, function(rdMap) {
                    dialogMessageLayoutContent.push(_createInvalidItemLayout(rb.getText("ModelImplementation.RunInstanceInput.RiskDataObject.Selector.Label.txt"), rdMap.dataset.label));
                });
            }
            if(invalidDatasets.portfolio) {
                dialogMessageLayoutContent.push(_createInvalidItemLayout(rb.getText("ModelImplementation.RunInstanceInput.PortfolioDataSelector.txt"), invalidDatasets.portfolio.label));
            }
            if(invalidDatasets.economic) {
                var name = (invalidDatasets.economic.category === "folder") ? rb.getText("ModelImplementation.RunInstanceInput.EconomicDataSelector.folder.txt") :
                    rb.getText("ModelImplementation.RunInstanceInput.EconomicDataSelector.library.txt");
                dialogMessageLayoutContent.push(_createInvalidItemLayout(name, invalidDatasets.economic.label));
            }
            if(invalidDatasets.economicSimulation) {
                dialogMessageLayoutContent.push(_createInvalidItemLayout(rb.getText("ModelImplementation.RunInstanceInput.EconomicSimulationDataSelector.txt"), invalidDatasets.economicSimulation.label));
            }
            if(invalidDatasets.counterparty) {
                dialogMessageLayoutContent.push(_createInvalidItemLayout(rb.getText("ModelImplementation.RunInstanceInput.CounterpartyDataSelector.txt"), invalidDatasets.counterparty.label));
            }
            if(invalidDatasets.mitigation) {
                dialogMessageLayoutContent.push(_createInvalidItemLayout(rb.getText("ModelImplementation.RunInstanceInput.MitigationDataSelector.txt"), invalidDatasets.mitigation.label));
            }

            var text = new sas.hc.m.Text({text: isSourceCube ? rb.getText("ModelImplementation.RunInstanceInput.MissingSourceCubeItems.txt") : rb.getText("ModelImplementation.RunInstanceInput.MissingRunItems.txt")});
            text.addStyleClass("mipNameValueListLayout");
            var invalidDataDialog = new sas.hc.m.Dialog({
                icon: sas.icons.HC.WARNINGSTATUS,
                title: isSourceCube ? rb.getText("ModelImplementation.RunInstanceInput.MissingSourceCubeItems.title.txt") : rb.getText("ModelImplementation.RunInstanceInput.MissingRunItems.title.txt"),
                type: sap.m.DialogType.Message,
                content: [
                    new sas.hc.ui.layout.VerticalLayout({
                        content: [
                            text,
                            dialogMessageLayoutContent
                        ]
                    })
                ],

                endButton: new sas.hc.m.Button({
                    text: rb.getText("button.close.action.label"),
                    type: sap.m.ButtonType.Accept,
                    press: function () {
                        invalidDataDialog.close();
                    }
                }),

                afterClose: function() {
                    invalidDataDialog.destroy();
                }
            });
            if (parentView) {
                parentView.addDependent(invalidDataDialog);
            }
            invalidDataDialog.open();
        },

        /**
         * Utility valueState Binding creation method - can be used when you want to bind a control's value state to one or more paths.
         *
         * When adding an array of paths as param, pay attention to 1st one wins rule here.
         * i.e. add the error paths first before adding warning paths
         * also in between multiple error paths, decide the order based on the weight of the error
         * selectedErrorPath : this can a single path or an array of paths
         * Returns Binding object {parts: Binding path array, formatter: formatter function}
         */
        computeValueStateBinding: function(selectedErrorPath){
            if(!selectedErrorPath){ //For the consumers who don't need to set value state based on error
                return null;
            }
            var parts = [];
            if(_.isArray(selectedErrorPath)) {
                _.each(selectedErrorPath, function (p) {
                    parts.push({path: p});
                });
            }else {
                parts.push({path: selectedErrorPath});
            }
            return {
                parts: parts,
                formatter: function(){
                    //Find the first error or warning value state
                    var errorOrWarning = _.find(arguments, function(arg){
                        return !!arg && arg.toLowerCase() !== sap.ui.core.ValueState.None.toLowerCase();
                    });
                    if(errorOrWarning){ //Use the error or warning state found
                        switch(errorOrWarning.toLowerCase()){
                            case sap.ui.core.ValueState.Error.toLowerCase():
                                return sap.ui.core.ValueState.Error;
                            case sap.ui.core.ValueState.Warning.toLowerCase():
                                return sap.ui.core.ValueState.Warning;
                            default:
                                return sap.ui.core.ValueState.None;
                        }
                    }else{
                        return sap.ui.core.ValueState.None;
                    }
                }
            };
        },

        /**
         * Utility valueStateText Binding creation method - can be used when you want to bind a control's valueState and valueStateText to one or more paths.
         *
         * For value state text binding, it picks up from the error path/paths passed for valueState binding
         * with the assumption of value state text path being = error path + the word "Text"
         * For example if the error paths for valueStates are set to ["/evaluationError", "duplicateOutVarsError"]
         * then it is assumed the paths for valueStateTexts are : ["/evaluationErrorText", "duplicateOutVarsErrorText"]
         * selectedErrorPath : this can a single path or an array of paths
         * Returns Binding object{parts: Binding path array, formatter: formatter function}
         */
        computeValueStateTextBinding: function(selectedErrorPath){
            if(!selectedErrorPath){ //For the consumers who don't need to set value state based on error
                return null;
            }
            var parts = [];
            if(_.isArray(selectedErrorPath)) {
                _.each(selectedErrorPath, function (p) {
                    parts.push({path: p+"Text"});
                });
            }else {
                parts.push({path: selectedErrorPath+"Text"});
            }
            return {
                parts: parts,
                formatter: function(){
                    var errorText = _.find(arguments, function(arg){ //1st valid value state text wins
                        return !!arg;
                    });
                    return errorText || "";
                }
            };
        },

        /**
         * Find the parent view of the current step in the run wizard
         * @returns {Object} view or null
         */
        getParentOfStepView: function() {
            var parentView = null;
            for (var p=0; p<ModelImplementationConstants.RUN_VIEW_IDS.length; p++) {
                var id = ModelImplementationConstants.RUN_VIEW_IDS[p];
                parentView = sap.ui.getCore().byId(id);
                if (parentView) {
                    break;
                }
            }
            return parentView;
        }
    };
})();