




 /*--------------------------------------------------------------
  * Copyright 2015, SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
  *
  * NAME:      rskmimpl_jobstatus.sas
  * PURPOSE:   A simple wrapper macro to improve the readability
  *            of the mim generated code.  This macro allows
  *            us to not specify the default settings for
  *            mim on the job status elements.
  *
  * USAGE:
  *            rskmimpl_jobstatus(
  *               mimpl_jobstep=3,
  *               mimpl_action= generate_summary_report,
  *               mimpl_desc="Creating results summary .");
  *
  * PARAMETERS:
  *    THESE PARAMETERS ARE REQUIRED
  *    mimpl_jobstep   : specifies the step number
  *    mimpl_action    : specifies what the action is 
  *                      for that step. This action maps
  *                      to a key in the UI to determine
  *                      client side status messages.
  *
  *    THESE PARAMETERS ARE OPTIONAL
  *    mimpl_desc     : a default step description text
  *
  *-------------------------------------------------------------*/


%macro rskmimpl_jobstatus(
      mimpl_jobstep=, mimpl_action=,  mimpl_desc="",
      mimpl_jobds=miplogs.runstatus, mimpl_jtask="RUN_EXECUTION", 
      mimpl_totalsteps=4, mimpl_jfailure=0,
      mimpl_error_abend=Yes,
      mimpl_check_jobds=No,
      mimpl_run_type=
      );

    
%if (%sysfunc(exist(&mimpl_jobds)) ne 1 ) %then %do;
   %hpr_prog_ds_create(HPR_DS=&mimpl_jobds);
%end;

%local isquote;

%let isquote = %bquote(%substr(&mimpl_jtask,1,1));
%if %bquote(&isquote) ne %str(%') and %bquote(&isquote) ne %str(%") %then %do;
   %let mimpl_jtask = "&mimpl_jtask";
%end;

%let isquote = %bquote(%substr(&mimpl_action,1,1));
%if %bquote(&isquote) ne %str(%') and %bquote(&isquote) ne %str(%") %then %do;
   %let mimpl_action = "&mimpl_action";
%end;

%if %length(&mimpl_desc) = 0 %then %do;
   %let mimpl_desc = "";
%end;

%let isquote = %bquote(%substr(&mimpl_desc,1,1));
%if %bquote(&isquote) ne %str(%') and %bquote(&isquote) ne %str(%") %then %do;
   %let mimpl_desc = "&mimpl_desc";
%end;

%if &mimpl_jfailure=0 & &SYSERR^=0 & &SYSERR ^= 4 %then %do;       
    %let mimpl_jfailure=&SYSERR;
%end; 
%else %if &mimpl_jfailure=0 & %symexist(rd_riskerr) %then %do;
    %if &rd_riskerr ne -1 & &rd_riskerr ne 0 %then %let mimpl_jfailure = &rd_riskerr;
%end;

%if &mimpl_jfailure=0 & %upcase(&mimpl_check_jobds) eq YES %then %do;
    %rskmimpl_check_status_ds(RSKMIMPL_STATUS_DS = &mimpl_jobds, 
                              RSKMIMPL_RUN_TYPE = &mimpl_run_type,
                              RSKMIMPL_FAIL_FLAG = mimpl_jfailure
                              );
%end;

%hpr_prog_ds_append(HPR_DS=&mimpl_jobds, HPR_TASK_QUOTED=&mimpl_jtask, 
                    HPR_STEP_CNT=&mimpl_jobstep, HPR_TOTAL_CNT=&mimpl_totalsteps,
                    HPR_ACTION_QUOTED=&mimpl_action, HPR_DESC_QUOTED=&mimpl_desc,
                    HPR_FAIL=&mimpl_jfailure
);

%if &mimpl_jfailure ne 0 & %upcase(&mimpl_error_abend) eq YES %then %do;
    %abort cancel;
%end;

%mend rskmimpl_jobstatus;
